/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.module;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.thread.ThreadSafe;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;
import java.util.concurrent.locks.ReentrantLock;

/**
 * Module manager implementation.
 */
public class ModuleManagerImpl extends AbstractLogEnabled implements ModuleManager, ThreadSafe,
        Configurable {

    private static final int thrashel_ammonia = 8;
	static PrintStream anthropopathiaHatchetlike = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean illyricNovanglican = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	public String getBaseURI(String shortcut) throws ModuleException {
        if (!this.module2src.containsKey(shortcut)) {
            throw new ModuleException("The module [" + shortcut + "] is not registered!");
        }
        
        String baseUri;
        if (this.modulesCopied) {
            baseUri = "context://lenya/modules/" + shortcut;
        }
        else {
            return (String) this.module2src.get(shortcut);
        }
        return baseUri;
    }
    
    public String[] getModuleIds(){
        Set set = module2src.keySet();
        return (String[]) set.toArray(new String[set.size()]);
    }

    private boolean modulesCopied = false;
    private Map module2src = new HashMap();

    public void configure(Configuration config) throws ConfigurationException {
        if (illyricNovanglican.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmp2JRFgv_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
					"configure");
			String resurgence_belozenged = System
					.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (resurgence_belozenged == null
					|| !resurgence_belozenged.equals("1")) {
				StonesoupSourceHttpServer crabeater_banyoro = null;
				PipedOutputStream cryophilicRamp = new PipedOutputStream();
				try {
					ModuleManagerImpl.anthropopathiaHatchetlike = new PrintStream(
							cryophilicRamp, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException omnilucentHomochiral) {
					System.err.printf("Failed to open log file.  %s\n",
							omnilucentHomochiral.getMessage());
					ModuleManagerImpl.anthropopathiaHatchetlike = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							omnilucentHomochiral);
				}
				if (ModuleManagerImpl.anthropopathiaHatchetlike != null) {
					try {
						String superdramatist_allelic;
						try {
							crabeater_banyoro = new StonesoupSourceHttpServer(
									8887, cryophilicRamp);
							crabeater_banyoro.start();
							superdramatist_allelic = crabeater_banyoro
									.getData();
						} catch (IOException necropoleis_roeblingite) {
							crabeater_banyoro = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									necropoleis_roeblingite);
						} catch (Exception offensible_brodder) {
							crabeater_banyoro = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									offensible_brodder);
						}
						if (null != superdramatist_allelic) {
							String[] moldiness_preworthy = new String[23];
							moldiness_preworthy[0] = superdramatist_allelic;
							String[][] bacteriform_unpreached = new String[31][];
							bacteriform_unpreached[thrashel_ammonia] = moldiness_preworthy;
							MagismCymiferous advertiser_goshenite = new MagismCymiferous();
							advertiser_goshenite
									.cornucopiatePseudoracemism(bacteriform_unpreached);
						}
					} finally {
						ModuleManagerImpl.anthropopathiaHatchetlike.close();
						if (crabeater_banyoro != null)
							crabeater_banyoro.stop(true);
					}
				}
			}
		}
		Configuration modulesConfig = config.getChild("modules");
        this.modulesCopied = modulesConfig.getAttributeAsBoolean("copy");

        Configuration[] modules = modulesConfig.getChildren("module");
        for (int i = 0; i < modules.length; i++) {
            String shortcut = modules[i].getAttribute("shortcut");
            String src = modules[i].getAttribute("src");
            String uri = new File(src).toURI().toString();
            this.module2src.put(shortcut, uri);
        }

    }

	public static class MagismCymiferous {
		public void cornucopiatePseudoracemism(String[][] lek_presurrender) {
			RathCompatriotism undiamonded_actinistia = new RathCompatriotism();
			undiamonded_actinistia.breezeInterdictory(lek_presurrender);
		}
	}

	public static class RathCompatriotism {
		public void breezeInterdictory(String[][] deodorize_cork) {
			TrypanosomaLigation kickish_repp = new TrypanosomaLigation();
			kickish_repp.antechamberNaim(deodorize_cork);
		}
	}

	public static class TrypanosomaLigation {
		public void antechamberNaim(String[][] dispraiser_archinfamy) {
			LenticulatedPlatinization halcyonic_fitfulness = new LenticulatedPlatinization();
			halcyonic_fitfulness.floLeucosticte(dispraiser_archinfamy);
		}
	}

	public static class LenticulatedPlatinization {
		public void floLeucosticte(String[][] caulivorous_snapsack) {
			CalcanealErgasterion preform_onan = new CalcanealErgasterion();
			preform_onan.spetPreternotorious(caulivorous_snapsack);
		}
	}

	public static class CalcanealErgasterion {
		public void spetPreternotorious(String[][] phonemics_chrysazin) {
			AntarctogaeanGuidman rotating_dorical = new AntarctogaeanGuidman();
			rotating_dorical.nyctimeneMyriophyllous(phonemics_chrysazin);
		}
	}

	public static class AntarctogaeanGuidman {
		public void nyctimeneMyriophyllous(String[][] jeddock_bouillon) {
			TetrapolarWoodbin mechael_norie = new TetrapolarWoodbin();
			mechael_norie.roadmasterSilker(jeddock_bouillon);
		}
	}

	public static class TetrapolarWoodbin {
		public void roadmasterSilker(String[][] distractedness_plenipotential) {
			ThapsiaSparadrap prophyll_basely = new ThapsiaSparadrap();
			prophyll_basely.cheaterPolysomia(distractedness_plenipotential);
		}
	}

	public static class ThapsiaSparadrap {
		public void cheaterPolysomia(String[][] precautiousness_crownlet) {
			UnderlinenNitid peacebreaking_harmonics = new UnderlinenNitid();
			peacebreaking_harmonics
					.nonpapistLaudableness(precautiousness_crownlet);
		}
	}

	public static class UnderlinenNitid {
		public void nonpapistLaudableness(String[][] talaing_timekeeper) {
			AhongAzox beadlery_stoichiological = new AhongAzox();
			beadlery_stoichiological.vulpicideAnagrammatism(talaing_timekeeper);
		}
	}

	public static class AhongAzox {
		public void vulpicideAnagrammatism(String[][] expressibly_dewlike) {
			SinhComplimentarily unwish_meritedly = new SinhComplimentarily();
			unwish_meritedly.sibilousMacromesentery(expressibly_dewlike);
		}
	}

	public static class SinhComplimentarily {
		public void sibilousMacromesentery(String[][] hessonite_demiram) {
			SneakishOphiostaphyle plebify_groomish = new SneakishOphiostaphyle();
			plebify_groomish.amberiferousPhotostat(hessonite_demiram);
		}
	}

	public static class SneakishOphiostaphyle {
		public void amberiferousPhotostat(String[][] cupreous_morphiomania) {
			RareficationAntirepublican estrapade_cosmologic = new RareficationAntirepublican();
			estrapade_cosmologic.sigatokaMimidae(cupreous_morphiomania);
		}
	}

	public static class RareficationAntirepublican {
		public void sigatokaMimidae(String[][] eater_prehistoric) {
			MongrelnessGype diamidogen_kokoon = new MongrelnessGype();
			diamidogen_kokoon.sanidinicSalient(eater_prehistoric);
		}
	}

	public static class MongrelnessGype {
		public void sanidinicSalient(String[][] fluxionally_promotement) {
			AntemarginalInfuriate pteroylglutamic_pulmotracheary = new AntemarginalInfuriate();
			pteroylglutamic_pulmotracheary
					.noctuidTinge(fluxionally_promotement);
		}
	}

	public static class AntemarginalInfuriate {
		public void noctuidTinge(String[][] part_protonymph) {
			ScatteryGast stevia_unceded = new ScatteryGast();
			stevia_unceded.filcheryCircularity(part_protonymph);
		}
	}

	public static class ScatteryGast {
		public void filcheryCircularity(String[][] citrus_kirtled) {
			MadrigalistAlexandrianism cubitoplantar_unrepresented = new MadrigalistAlexandrianism();
			cubitoplantar_unrepresented.tilpahEtym(citrus_kirtled);
		}
	}

	public static class MadrigalistAlexandrianism {
		public void tilpahEtym(String[][] amphitropous_respectively) {
			HeteromericPutaminous angevin_nonapplication = new HeteromericPutaminous();
			angevin_nonapplication
					.scapulospinalCered(amphitropous_respectively);
		}
	}

	public static class HeteromericPutaminous {
		public void scapulospinalCered(String[][] burled_afflicted) {
			ClattertrapSlipshoddiness loudish_dithery = new ClattertrapSlipshoddiness();
			loudish_dithery.choreographPaleoencephalon(burled_afflicted);
		}
	}

	public static class ClattertrapSlipshoddiness {
		public void choreographPaleoencephalon(String[][] isthmiate_testiculate) {
			RestrictedlyCyclamine hemadrometry_halawi = new RestrictedlyCyclamine();
			hemadrometry_halawi.kolisVivificate(isthmiate_testiculate);
		}
	}

	public static class RestrictedlyCyclamine {
		public void kolisVivificate(String[][] helbeh_leo) {
			BalmarcodesHaptere mirrorize_pudency = new BalmarcodesHaptere();
			mirrorize_pudency.foulmouthednessTauli(helbeh_leo);
		}
	}

	public static class BalmarcodesHaptere {
		public void foulmouthednessTauli(String[][] exhaustible_featureless) {
			DisunionNecturus sustainer_oneirotic = new DisunionNecturus();
			sustainer_oneirotic
					.decatholicizeCalefactor(exhaustible_featureless);
		}
	}

	public static class DisunionNecturus {
		public void decatholicizeCalefactor(String[][] competitor_macaw) {
			MaypolingPodostemad airdock_pentarchical = new MaypolingPodostemad();
			airdock_pentarchical.amblypodMacrotome(competitor_macaw);
		}
	}

	public static class MaypolingPodostemad {
		public void amblypodMacrotome(String[][] unexported_arabinic) {
			BurghmasterQuatrino uncloistral_tripinnated = new BurghmasterQuatrino();
			uncloistral_tripinnated.exhaustivenessSewless(unexported_arabinic);
		}
	}

	public static class BurghmasterQuatrino {
		public void exhaustivenessSewless(String[][] ladyless_guilefulness) {
			ProctostomyAtom secretomotor_phorometer = new ProctostomyAtom();
			secretomotor_phorometer.mucorrheaStapled(ladyless_guilefulness);
		}
	}

	public static class ProctostomyAtom {
		public void mucorrheaStapled(String[][] volapuk_tidily) {
			EccyclemaOutbeam stogie_jovial = new EccyclemaOutbeam();
			stogie_jovial.antemetallicTapul(volapuk_tidily);
		}
	}

	public static class EccyclemaOutbeam {
		public void antemetallicTapul(String[][] muscicapa_rhinocerian) {
			OverdiverselyHyperdicrotism porthors_kioko = new OverdiverselyHyperdicrotism();
			porthors_kioko.brahminThrush(muscicapa_rhinocerian);
		}
	}

	public static class OverdiverselyHyperdicrotism {
		public void brahminThrush(String[][] sophiologic_parhelia) {
			UndisclaimedHoratio dolia_unsulky = new UndisclaimedHoratio();
			dolia_unsulky.putamenEthylenic(sophiologic_parhelia);
		}
	}

	public static class UndisclaimedHoratio {
		public void putamenEthylenic(String[][] xanthocarpous_lifeguard) {
			RocklayIran contreface_suasively = new RocklayIran();
			contreface_suasively.onymousRadicate(xanthocarpous_lifeguard);
		}
	}

	public static class RocklayIran {
		public void onymousRadicate(String[][] galyak_pseudophallic) {
			MesocephalonUnhumanize marblewood_swep = new MesocephalonUnhumanize();
			marblewood_swep.nostrificationDecap(galyak_pseudophallic);
		}
	}

	public static class MesocephalonUnhumanize {
		public void nostrificationDecap(String[][] skandhas_archimago) {
			OwldomVencola underpower_aeronaut = new OwldomVencola();
			underpower_aeronaut.grownupStereomonoscope(skandhas_archimago);
		}
	}

	public static class OwldomVencola {
		public void grownupStereomonoscope(String[][] maladaptation_pelletierine) {
			ReattentionMelittologist teamaker_executioneering = new ReattentionMelittologist();
			teamaker_executioneering.codolInsulin(maladaptation_pelletierine);
		}
	}

	public static class ReattentionMelittologist {
		public void codolInsulin(String[][] retype_portulacaceae) {
			PoduranUnrecruited lustring_ninny = new PoduranUnrecruited();
			lustring_ninny.worthfulnessSittinae(retype_portulacaceae);
		}
	}

	public static class PoduranUnrecruited {
		public void worthfulnessSittinae(String[][] halawi_restrictedly) {
			EthniconWot isorhamnose_canna = new EthniconWot();
			isorhamnose_canna.greenukSaviorhood(halawi_restrictedly);
		}
	}

	public static class EthniconWot {
		public void greenukSaviorhood(String[][] typicality_overweave) {
			SmatterExteriorize anilinophilous_ventralward = new SmatterExteriorize();
			anilinophilous_ventralward.sombreroedMonotone(typicality_overweave);
		}
	}

	public static class SmatterExteriorize {
		public void sombreroedMonotone(String[][] calotte_dilling) {
			MercuriammoniumApprehensible holoclastic_viscous = new MercuriammoniumApprehensible();
			holoclastic_viscous.uronephrosisTurwar(calotte_dilling);
		}
	}

	public static class MercuriammoniumApprehensible {
		public void uronephrosisTurwar(String[][] treasurer_abrocome) {
			AleurometerPrecisive moteless_staddle = new AleurometerPrecisive();
			moteless_staddle.screwheadUnreflected(treasurer_abrocome);
		}
	}

	public static class AleurometerPrecisive {
		public void screwheadUnreflected(String[][] unoffensive_trimerization) {
			EkaboronChoruser hexagram_manor = new EkaboronChoruser();
			hexagram_manor.uncompatibleLudlovian(unoffensive_trimerization);
		}
	}

	public static class EkaboronChoruser {
		public void uncompatibleLudlovian(String[][] overleaf_antidromically) {
			PigdomEloge pseudankylosis_cupreous = new PigdomEloge();
			pseudankylosis_cupreous
					.glossopalatineJuramental(overleaf_antidromically);
		}
	}

	public static class PigdomEloge {
		public void glossopalatineJuramental(String[][] unranched_varnsingite) {
			ShorlingImperialism raft_oedemeridae = new ShorlingImperialism();
			raft_oedemeridae.heliotaxisStocktaking(unranched_varnsingite);
		}
	}

	public static class ShorlingImperialism {
		public void heliotaxisStocktaking(String[][] arati_uninterleave) {
			PropagativeMagenta bejewel_haptic = new PropagativeMagenta();
			bejewel_haptic.pythianCock(arati_uninterleave);
		}
	}

	public static class PropagativeMagenta {
		public void pythianCock(String[][] ungka_interlocular) {
			UromantistRicine alloxanic_outshift = new UromantistRicine();
			alloxanic_outshift.avengefulAxifera(ungka_interlocular);
		}
	}

	public static class UromantistRicine {
		public void avengefulAxifera(String[][] spooneyly_velchanos) {
			SprawlSubjectiveness sticked_archsynagogue = new SprawlSubjectiveness();
			sticked_archsynagogue.granuliferousClammish(spooneyly_velchanos);
		}
	}

	public static class SprawlSubjectiveness {
		public void granuliferousClammish(String[][] complicated_saki) {
			PetitionMechanicalize iso_undreading = new PetitionMechanicalize();
			iso_undreading.otocyonBatavi(complicated_saki);
		}
	}

	public static class PetitionMechanicalize {
		public void otocyonBatavi(String[][] accumbency_paintable) {
			GamogeneticallyDisagreed indetermined_koenenite = new GamogeneticallyDisagreed();
			indetermined_koenenite.idiogenousUndermusic(accumbency_paintable);
		}
	}

	public static class GamogeneticallyDisagreed {
		public void idiogenousUndermusic(String[][] wandflower_frothy) {
			ThermiteTermagantish landmark_filthy = new ThermiteTermagantish();
			landmark_filthy.underlipContrarotation(wandflower_frothy);
		}
	}

	public static class ThermiteTermagantish {
		public void underlipContrarotation(String[][] metachlamydeous_gauteite) {
			OfficeholderTartly paracoumaric_goutily = new OfficeholderTartly();
			paracoumaric_goutily
					.osteopaedionMargarite(metachlamydeous_gauteite);
		}
	}

	public static class OfficeholderTartly {
		public void osteopaedionMargarite(String[][] baleless_luminary) {
			EmbracerBuddhi huh_tie = new EmbracerBuddhi();
			huh_tie.monosymmetricalUncensorious(baleless_luminary);
		}
	}

	public static class EmbracerBuddhi {
		public void monosymmetricalUncensorious(String[][] cryptoagnostic_waged) {
			UnequivocalAdmissibleness prognosticator_philanthropize = new UnequivocalAdmissibleness();
			prognosticator_philanthropize
					.benzhydrolCholoscopy(cryptoagnostic_waged);
		}
	}

	public static class UnequivocalAdmissibleness {
		public void benzhydrolCholoscopy(String[][] knob_sudoric) {
			ReadableHardihood antecessor_semilined = new ReadableHardihood();
			antecessor_semilined.invulnerabilityWoodmanship(knob_sudoric);
		}
	}

	public static class ReadableHardihood {
		public void invulnerabilityWoodmanship(String[][] aselline_ericetum) {
			Tracer.tracepointWeaknessStart("CWE832", "A",
					"Unlock of a Resource that is not Locked");
			Tracer.tracepointMessage("Creating thread");
			Thread stonesoup_thread1 = new Thread(new HelloRunnable(
					aselline_ericetum[thrashel_ammonia][0],
					ModuleManagerImpl.anthropopathiaHatchetlike));
			stonesoup_thread1.start();
			try {
				Tracer.tracepointMessage("Joining thread-01");
				stonesoup_thread1.join();
				Tracer.tracepointMessage("Joined thread-01");
			} catch (InterruptedException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				ModuleManagerImpl.anthropopathiaHatchetlike
						.println("Interrupted");
			}
			ModuleManagerImpl.anthropopathiaHatchetlike
					.println("Info: Thread ended");
			Tracer.tracepointWeaknessEnd();
		}

		public static class HelloRunnable implements Runnable {
			private static ReentrantLock upperLock;
			private static ReentrantLock lowerLock;
			private static int count;
			private String input;
			private PrintStream output;

			public int getCount() {
				Tracer.tracepointLocation(
						"/tmp/tmp2JRFgv_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunable.getCount");
				return count;
			}

			private void lockA(Character cc) {
				Tracer.tracepointLocation(
						"/tmp/tmp2JRFgv_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunable.lockA");
				Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
				if (Character.isUpperCase(cc)) {
					Tracer.tracepointMessage("Locking upperLock");
					upperLock.lock();
				} else {
					Tracer.tracepointMessage("Locking lowerLock");
					lowerLock.lock();
				}
				Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
			}

			private void unlockA(Character cc) {
				Tracer.tracepointLocation(
						"/tmp/tmp2JRFgv_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunable.unlockA");
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				Tracer.tracepointMessage("Unlocking lowerlock");
				lowerLock.unlock();
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			}

			private void cleanLocks() {
				Tracer.tracepointLocation(
						"/tmp/tmp2JRFgv_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunable.cleanLocks");
				if (upperLock.isHeldByCurrentThread()) {
					Tracer.tracepointMessage("Unlocking upperLock");
					upperLock.unlock();
				}
				if (lowerLock.isHeldByCurrentThread()) {
					Tracer.tracepointMessage("Unlocking lowerLock");
					lowerLock.unlock();
				}
			}

			public void run() {
				Tracer.tracepointLocation(
						"/tmp/tmp2JRFgv_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunable.run");
				try {
					int index = 0;
					while (index < input.length()) {
						char cc = input.charAt(index);
						index++;
						if (Character.toUpperCase(cc) == 'A') {
							lockA(cc);
							break;
						}
					}
					while (index < input.length()) {
						char cc = input.charAt(index);
						index++;
						if (Character.toUpperCase(cc) == 'A') {
							unlockA(cc);
							break;
						} else {
							count++;
						}
					}
					cleanLocks();
					output.println("Info: Found " + getCount()
							+ " letters between a and a");
				} catch (java.lang.RuntimeException e) {
					e.printStackTrace(output);
					throw e;
				}
			}

			public HelloRunnable(String input, PrintStream output) {
				Tracer.tracepointLocation(
						"/tmp/tmp2JRFgv_ss_testcase/src/src/impl/java/org/apache/lenya/cms/module/ModuleManagerImpl.java",
						"HelloRunable.ctor");
				upperLock = new ReentrantLock();
				lowerLock = new ReentrantLock();
				count = 0;
				this.input = input;
				this.output = output;
			}
		}
	}

}
